package java.net;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileDescriptor;

public abstract class SocketImpl implements SocketOptions
{
    Socket socket;
    ServerSocket serverSocket;
    protected FileDescriptor fd;
    protected InetAddress address;
    protected int port;
    protected int localport;
    
    public SocketImpl() {
        this.socket = null;
        this.serverSocket = null;
    }
    
    protected abstract void create(boolean p0) throws IOException;
    
    protected abstract void connect(String p0, int p1) throws IOException;
    
    protected abstract void connect(InetAddress p0, int p1) throws IOException;
    
    protected abstract void connect(SocketAddress p0, int p1) throws IOException;
    
    protected abstract void bind(InetAddress p0, int p1) throws IOException;
    
    protected abstract void listen(int p0) throws IOException;
    
    protected abstract void accept(SocketImpl p0) throws IOException;
    
    protected abstract InputStream getInputStream() throws IOException;
    
    protected abstract OutputStream getOutputStream() throws IOException;
    
    protected abstract int available() throws IOException;
    
    protected abstract void close() throws IOException;
    
    protected void shutdownInput() throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    protected void shutdownOutput() throws IOException {
        throw new IOException("Method not implemented!");
    }
    
    protected FileDescriptor getFileDescriptor() {
        return this.fd;
    }
    
    protected InetAddress getInetAddress() {
        return this.address;
    }
    
    protected int getPort() {
        return this.port;
    }
    
    protected boolean supportsUrgentData() {
        return false;
    }
    
    protected abstract void sendUrgentData(int p0) throws IOException;
    
    protected int getLocalPort() {
        return this.localport;
    }
    
    void setSocket(Socket socket) {
        this.socket = socket;
    }
    
    Socket getSocket() {
        return this.socket;
    }
    
    void setServerSocket(ServerSocket serverSocket) {
        this.serverSocket = serverSocket;
    }
    
    ServerSocket getServerSocket() {
        return this.serverSocket;
    }
    
    @Override
    public String toString() {
        return "Socket[addr=" + this.getInetAddress() + ",port=" + this.getPort() + ",localport=" + this.getLocalPort() + "]";
    }
    
    void reset() throws IOException {
        this.address = null;
        this.port = 0;
        this.localport = 0;
    }
    
    protected void setPerformancePreferences(int n, int n2, int n3) {
    }
    
     <T> void setOption(SocketOption<T> socketOption, T t) throws IOException {
        if (socketOption == StandardSocketOptions.SO_KEEPALIVE) {
            this.setOption(8, t);
        }
        else if (socketOption == StandardSocketOptions.SO_SNDBUF) {
            this.setOption(4097, t);
        }
        else if (socketOption == StandardSocketOptions.SO_RCVBUF) {
            this.setOption(4098, t);
        }
        else if (socketOption == StandardSocketOptions.SO_REUSEADDR) {
            this.setOption(4, t);
        }
        else if (socketOption == StandardSocketOptions.SO_LINGER) {
            this.setOption(128, t);
        }
        else if (socketOption == StandardSocketOptions.IP_TOS) {
            this.setOption(3, t);
        }
        else if (socketOption == StandardSocketOptions.TCP_NODELAY) {
            this.setOption(1, t);
        }
        else {
            throw new UnsupportedOperationException("unsupported option");
        }
    }
    
     <T> T getOption(SocketOption<T> socketOption) throws IOException {
        if (socketOption == StandardSocketOptions.SO_KEEPALIVE) {
            return this.getOption(8);
        }
        if (socketOption == StandardSocketOptions.SO_SNDBUF) {
            return this.getOption(4097);
        }
        if (socketOption == StandardSocketOptions.SO_RCVBUF) {
            return this.getOption(4098);
        }
        if (socketOption == StandardSocketOptions.SO_REUSEADDR) {
            return this.getOption(4);
        }
        if (socketOption == StandardSocketOptions.SO_LINGER) {
            return this.getOption(128);
        }
        if (socketOption == StandardSocketOptions.IP_TOS) {
            return this.getOption(3);
        }
        if (socketOption == StandardSocketOptions.TCP_NODELAY) {
            return this.getOption(1);
        }
        throw new UnsupportedOperationException("unsupported option");
    }
}
